@@ -1,11 +1,21 @@
 """Generic utilities."""
-from typing import Any, Dict, Iterable, List, Optional, TypeVar
+from typing import (
+    Any, Dict, Hashable, Iterable, List, Optional, Sequence, TypeVar,
+)
 
 import click
 
+from cloup.typing import MISSING, Possibly
+
 click_version_tuple = click.__version__.split('.')
 
 T = TypeVar('T')
+K = TypeVar('K', bound=Hashable)
+V = TypeVar('V')
+
+
+def pick_non_missing(d: Dict[K, Possibly[V]]) -> Dict[K, V]:
+    return {key: val for key, val in d.items() if val is not MISSING}
 
 
 def class_name(obj):
@@ -69,16 +79,18 @@ def pluralize(
     return many.format(count=count)
 
 
-def coalesce(*values: Optional[T], default=None) -> Optional[T]:
-    """Returns the first value that is not None (or ``default`` if no such value exists).
-    Inspired by the homonym SQL function."""
-    for val in values:
-        if val is not None:
-            return val
-    return default
+def first_not_none(*values: Optional[T]) -> Optional[T]:
+    """Returns the first value that is not None
+    (or ``default`` if no such value exists)."""
+    return next((val for val in values if val is not None), None)
 
 
-def listOfNotNone(iterable: Iterable[Optional[T]]) -> List[T]:
+def first_bool(*values: Optional[bool]) -> bool:
+    """Returns the first bool (or raises StopIteration if no bool is found)."""
+    return next(val for val in values if isinstance(val, bool))
+
+
+def pick_not_none(iterable: Iterable[Optional[T]]) -> List[T]:
     return [x for x in iterable if x is not None]
 
 
@@ -105,7 +117,7 @@ class FrozenSpaceMeta(type):
         raise Exception("you can't set attributes on this class")
 
     def asdict(cls) -> Dict[str, Any]:
-        return cls._dict   # type: ignore
+        return cls._dict  # type: ignore
 
     def __contains__(cls, item: str) -> bool:
         return item in cls.asdict()
@@ -122,7 +134,6 @@ class FrozenSpace(metaclass=FrozenSpaceMeta):
             "this class is just a namespace for constants, it's not instantiable.")
 
 
-def pop_many(d: dict, *keys: str) -> dict:
+def delete_keys(d: dict, keys: Sequence[str]) -> None:
     for key in keys:
-        d.pop(key)
-    return d
+        del d[key]
