@@ -1,14 +1,15 @@
 import abc
-from contextlib import contextmanager
 from typing import (
-    Callable, Iterable, Optional, Sequence, TypeVar, Union, overload
+    Callable, Iterable, Optional, Sequence, TypeVar, Union, overload,
 )
 
 import click
 from click import Context, Parameter
 
-from cloup._util import check_arg, class_name, make_one_line_repr, make_repr, pluralize
-from .exceptions import ConstraintViolated, UnsatisfiableConstraint
+from cloup._util import (
+    FrozenSpace, check_arg, class_name,
+    make_one_line_repr, make_repr, pluralize
+)
 from .common import (
     format_param_list,
     get_param_label,
@@ -17,10 +18,12 @@ from .common import (
     get_required_params,
     param_value_is_set,
 )
+from .exceptions import ConstraintViolated, UnsatisfiableConstraint
+from ..typing import Decorator, F
 
 Op = TypeVar('Op', bound='Operator')
 HelpRephraser = Callable[[Context, 'Constraint'], str]
-ErrorRephraser = Callable[[Context, 'Constraint', Sequence[Parameter]], str]
+ErrorRephraser = Callable[[ConstraintViolated], str]
 
 
 class Constraint(abc.ABC):
@@ -28,31 +31,30 @@ class Constraint(abc.ABC):
     A constraint that can be checked against an arbitrary collection of CLI
     parameters with respect to a specific :class:`click.Context` (which
     contains the values assigned to the parameters in ``ctx.params``).
+
+    .. versionchanged:: 0.9.0
+        calling a constraint, previously equivalent to :meth:`~Constraint.check`,
+        is now equivalent to calling :func:`cloup.constrained_params` with this
+        constraint as first argument.
     """
-    __check_consistency: bool = True
 
-    @classmethod
-    def must_check_consistency(cls) -> bool:
-        """Returns True if consistency checks are enabled."""
-        return cls.__check_consistency
+    @staticmethod
+    def must_check_consistency(ctx: click.Context) -> bool:
+        """Returns True if consistency checks are enabled.
 
-    @classmethod
-    def toggle_consistency_checks(cls, value: bool):
-        """Enables/disables consistency checks. Enabling means that:
-
-        - :meth:`check` will call :meth:`check_consistency`
-        - :class:`~cloup.ConstraintMixin` will call `check_consistency` on
-          constraints it is responsible for before parsing CLI arguments.
+        .. versionchanged:: 0.9.0
+            this method now a static method and takes a ``Context`` in input.
         """
-        cls.__check_consistency = value
+        return getattr(ctx, 'check_constraints_consistency', True)
 
-    @classmethod
-    @contextmanager
-    def consistency_checks_toggled(cls, value: bool):
-        value_to_restore = Constraint.__check_consistency
-        cls.__check_consistency = value
-        yield
-        cls.__check_consistency = value_to_restore
+    def __getattr__(self, attr):
+        removed_attrs = ('toggle_consistency_checks', 'consistency_checks_toggled')
+        if attr in removed_attrs:
+            raise Exception(
+                f'{attr} was removed in v0.9.0. You can now enable/disable consistency'
+                f'checks using the Context parameter check_constraints_consistency. '
+                f'Pass it as part of your context_settings.'
+            )
 
     @abc.abstractmethod
     def help(self, ctx: Context) -> str:
@@ -71,7 +73,8 @@ class Constraint(abc.ABC):
         depend on the values assigned to the parameters; therefore:
 
         - they can be performed before any parameter parsing
-        - they can be disabled in production (see :meth:`toggle_consistency_checks`)
+        - they can be disabled in production (setting
+          ``check_constraints_consistency=False`` in ``context_settings``)
 
         :param params: list of :class:`click.Parameter` instances
         :raises: :exc:`~cloup.constraints.errors.UnsatisfiableConstraint`
@@ -97,11 +100,11 @@ class Constraint(abc.ABC):
 
     @overload
     def check(self, params: Sequence[Parameter], ctx: Optional[Context] = None) -> None:
-        ...  # pragma: no cover
+        ...
 
     @overload
     def check(self, params: Iterable[str], ctx: Optional[Context] = None) -> None:
-        ...  # pragma: no cover
+        ...
 
     def check(self, params, ctx: Optional[Context] = None) -> None:
         """
@@ -114,7 +117,8 @@ class Constraint(abc.ABC):
         .. tip::
             By default :meth:`check_consistency` is called since it shouldn't
             have any performance impact. Nonetheless, you can disable it in
-            production passing ``False`` to :meth:`toggle_consistency_checks`.
+            production passing ``check_constraints_consistency=False`` as part
+            of your ``context_settings``.
 
         :param params: an iterable of parameter names or a sequence of
                        :class:`click.Parameter`
@@ -137,7 +141,7 @@ class Constraint(abc.ABC):
         params_objects = (ctx.command.get_params_by_name(params)
                           if isinstance(params[0], str)
                           else params)
-        if self.must_check_consistency():
+        if Constraint.must_check_consistency(ctx):
             self.check_consistency(params_objects)
         return self.check_values(params_objects, ctx)
 
@@ -146,16 +150,58 @@ class Constraint(abc.ABC):
         help: Union[None, str, HelpRephraser] = None,
         error: Union[None, str, ErrorRephraser] = None,
     ) -> 'Rephraser':
+        """
+        Overrides the help string and/or the error message of this constraint
+        wrapping it with a :class:`Rephraser`.
+
+        :param help:
+            if provided, overrides the help string of this constraint. It can be
+            a string or a function ``(ctx: Context, constr: Constraint) -> str``.
+            If you want to hide this constraint from the help, pass ``help=""``.
+        :param error:
+            if provided, overrides the error message of this constraint.
+            It can be:
+
+            - a string, eventually a ``format`` string supporting the replacement
+              fields described in :class:`ErrorFmt`.
+
+            - or a function ``(err: ConstraintViolated) -> str``; note that
+              a :class:`ConstraintViolated` error has fields for ``ctx``,
+              ``constraint`` and ``params``, so it's a complete description
+              of what happened.
+        """
         return Rephraser(self, help=help, error=error)
 
     def hidden(self) -> 'Rephraser':
         """Hides this constraint from the command help."""
         return Rephraser(self, help='')
 
-    def __call__(
-        self, param_names: Iterable[str], ctx: Optional[Context] = None
-    ) -> None:
-        return self.check(param_names, ctx=ctx)
+    def __call__(self, *param_adders: Decorator) -> Callable[[F], F]:
+        """Equivalent to calling :func:`cloup.constrained_params` with this
+        constraint as first argument.
+
+        .. versionchanged:: 0.9.0
+            this method, previously equivalent to :meth:`~Constraint.check`, is
+            now equivalent to calling :func:`cloup.constrained_params` with this
+            constraint as first argument.
+        """
+        from ._support import constrained_params
+        # TODO: remove this check in the future
+        if not callable(param_adders[0]):
+            from cloup import __version__
+            raise TypeError(f"""
+                all arguments of this method must be decorators like those returned
+                by cloup.argument() and cloup.option().
+
+                If you are trying to call a constraint to check it imperatively,
+                know that, since Cloup v0.9, calling a constraint has a completely
+                different semantics, see:
+
+                https://cloup.readthedocs.io/en/v{__version__}/pages/constraints.html#constraints-as-decorators
+
+                You can use the check() method to check a constraint imperatively.
+            """)
+        return constrained_params(self, *param_adders)
 
     def __or__(self, other: 'Constraint') -> 'Or':
         return Or(self, other)
@@ -217,7 +263,9 @@ class Or(Operator):
                 return c.check_values(params, ctx)
             except ConstraintViolated:
                 pass
-        raise ConstraintViolated.default(params, self.help(ctx), ctx=ctx)
+        raise ConstraintViolated.default(
+            self.help(ctx), ctx=ctx, constraint=self, params=params
+        )
 
     def __or__(self, other) -> 'Or':
         if isinstance(other, Or):
@@ -225,12 +273,38 @@ class Or(Operator):
         return Or(*self.constraints, other)
 
 
+class ErrorFmt(FrozenSpace):
+    """:class:`Rephraser` allows you to pass a ``format`` string as ``error``
+    argument; this class contains the "replacement fields" supported by such
+    format string. You can use them as following::
+
+        mutually_exclusive.rephrased(
+            error=f"{ErrorFmt.error}\\n"
+                  f"Some extra information here."
+        )
+    """
+
+    error = '{error}'
+    """Replaced by the original error message. Useful if all you want is to
+    append or prepend some extra info to the original error message."""
+
+    param_list = '{param_list}'
+    """Replaced by a 2-space indented list of the constrained parameters."""
+
+
 class Rephraser(Constraint):
     """A Constraint decorator that can override the help and/or the error
     message of the wrapped constraint.
 
-    This is useful also for defining new constraints.
-    See also :class:`WrapperConstraint`.
+    You'll rarely (if ever) use this class directly. In most cases, you'll use
+    the method :meth:`Constraint.rephrased`. Refer to it for more info.
+
+    .. seealso::
+
+        - :meth:`Constraint.rephrased` -- wraps a constraint with a ``Rephraser``.
+        - :class:`WrapperConstraint` -- alternative to ``Rephraser``.
+        - :class:`ErrorFmt` -- describes the keyword you can use in an error
+          format string.
     """
 
     def __init__(
@@ -252,15 +326,16 @@ class Rephraser(Constraint):
         else:
             return self._help(ctx, self._constraint)
 
-    def _get_rephrased_error(
-        self, ctx: Context, params: Sequence[Parameter]
-    ) -> Optional[str]:
+    def _get_rephrased_error(self, err: ConstraintViolated) -> Optional[str]:
         if self._error is None:
             return None
         elif isinstance(self._error, str):
-            return self._error.format(param_list=format_param_list(params))
+            return self._error.format(
+                error=str(err),
+                param_list=format_param_list(err.params),
+            )
         else:
-            return self._error(ctx, self._constraint, params)
+            return self._error(err)
 
     def check_consistency(self, params: Sequence[Parameter]) -> None:
         try:
@@ -272,10 +347,11 @@ class Rephraser(Constraint):
     def check_values(self, params: Sequence[Parameter], ctx: Context):
         try:
             return self._constraint.check_values(params, ctx)
-        except ConstraintViolated:
-            rephrased_error = self._get_rephrased_error(ctx, params)
+        except ConstraintViolated as err:
+            rephrased_error = self._get_rephrased_error(err)
             if rephrased_error:
-                raise ConstraintViolated(rephrased_error, ctx=ctx)
+                raise ConstraintViolated(
+                    rephrased_error, ctx=ctx, constraint=self, params=params)
             raise
 
     def __repr__(self):
@@ -334,6 +410,8 @@ class _RequireAll(Constraint):
                     many=f"the following parameters are required:\n"
                          f"{format_param_list(unset_params)}"),
                 ctx=ctx,
+                constraint=self,
+                params=params,
             )
 
 
@@ -363,7 +441,7 @@ class RequireAtLeast(Constraint):
             raise ConstraintViolated(
                 f"at least {n} of the following parameters must be set:\n"
                 f"{format_param_list(params)}",
-                ctx=ctx
+                ctx=ctx, constraint=self, params=params,
             )
 
     def __repr__(self):
@@ -393,7 +471,7 @@ class AcceptAtMost(Constraint):
             raise ConstraintViolated(
                 f"no more than {n} of the following parameters can be set:\n"
                 f"{format_param_list(params)}",
-                ctx=ctx,
+                ctx=ctx, constraint=self, params=params,
             )
 
     def __repr__(self):
@@ -420,7 +498,8 @@ class RequireExactly(WrapperConstraint):
                 zero='none of the following parameters must be set:\n',
                 many=f'exactly {n} of the following parameters must be set:\n'
             ) + format_param_list(params)
-            raise ConstraintViolated(reason, ctx=ctx)
+            raise ConstraintViolated(
+                reason, ctx=ctx, constraint=self, params=params)
 
 
 class AcceptBetween(WrapperConstraint):
@@ -447,19 +526,28 @@ require_all = _RequireAll()
 
 accept_none = AcceptAtMost(0).rephrased(
     help='all forbidden',
-    error='the following parameters should not be provided:\n{param_list}'
+    error=f'the following parameters should not be provided:\n'
+          f'{ErrorFmt.param_list}'
 )
 """Satisfied if none of the parameters is set. Useful only in conditional constraints."""
 
 all_or_none = (require_all | accept_none).rephrased(
     help='provide all or none',
-    error='the following parameters should be provided together (or none of '
-          'them should be provided):\n{param_list}',
+    error=f'the following parameters should be provided together (or none of '
+          f'them should be provided):\n'
+          f'{ErrorFmt.param_list}',
 )
 """Satisfied if either all or none of the parameters are set."""
 
 mutually_exclusive = AcceptAtMost(1).rephrased(
     help='mutually exclusive',
-    error='the following parameters are mutually exclusive:\n{param_list}'
+    error=f'the following parameters are mutually exclusive:\n'
+          f'{ErrorFmt.param_list}'
 )
 """Satisfied if at most one of the parameters is set."""
+
+require_any = RequireAtLeast(1)
+"""Alias for ``RequireAtLeast(1)``."""
+
+require_one = RequireExactly(1)
+"""Alias for ``RequireExactly(1)``."""
