@@ -1,14 +1,14 @@
-from pathlib import Path
 from pprint import pprint
 
 import click
 
 import cloup
 from cloup import argument, option, option_group
+from cloup.constraints import ErrorFmt, mutually_exclusive
 
 
 @cloup.command()
-@argument("script_path", type=Path, required=True)
+@argument("script_path", type=click.Path(), required=True)
 @argument("scene_names", required=False, nargs=-1)
 @option_group(
     "Global options",
@@ -81,13 +81,11 @@ from cloup import argument, option, option_group
     option(
         "-q", "--quality", default="h",
         type=click.Choice(["l", "m", "h", "p", "k"], case_sensitive=False),
-        help="""
-            Render quality at the follow resolution framerates, respectively:
-            854x480 30FPS,
-            1280x720 30FPS,
-            1920x1080 60FPS,
-            2560x1440 60FPS,
-            3840x2160 60FPS
+        help="""\b
+            Resolution and framerate of the render:
+            l = 854x480  30FPS,   m = 1280x720 30FPS,
+            h = 1920x1080 60FPS,  p = 2560x1440 60FPS,
+            k = 3840x2160 60FPS
             """,
     ),
     option(
@@ -98,10 +96,22 @@ from cloup import argument, option, option_group
         "--fps", "--frame_rate", "frame_rate", type=float,
         help="Render at this frame rate.",
     ),
-    option(
-        "--renderer",
-        type=click.Choice(["cairo", "opengl", "webgl"], case_sensitive=False),
-        help="Select a renderer for your Scene.",
+    mutually_exclusive.rephrased(
+        error=f"{ErrorFmt.error}.\nUse --renderer, the other two options are deprecated."
+    )(
+        option(
+            "--renderer",
+            type=click.Choice(["cairo", "opengl", "webgl"], case_sensitive=False),
+            help="Select a renderer for your Scene.",
+        ),
+        option(
+            "--use_opengl_renderer", is_flag=True, hidden=True,
+            help="(Deprecated) Use --renderer=opengl.",
+        ),
+        option(
+            "--use_cairo_renderer", is_flag=True, hidden=True,
+            help="(Deprecated) Use --renderer=cairo.",
+        ),
     ),
     option(
         "--webgl_renderer_path", type=click.Path(),
