@@ -1,12 +1,29 @@
 """Tests for the "option groups" feature/module."""
 from textwrap import dedent
+from typing import cast
 
+import click
 import pytest
 from click import pass_context
 
 import cloup
-from cloup import OptionGroup, option
-from tests.util import make_options, parametrize, noop
+from cloup import OptionGroup, option, option_group
+from cloup._util import pick_non_missing
+from cloup.typing import MISSING
+from cloup.constraints import RequireAtLeast, mutually_exclusive
+from tests.util import (make_options, new_dummy_func, parametrize, pick_first_bool)
+
+
+def test_error_message_if_first_arg_is_not_a_string():
+    with pytest.raises(
+        TypeError, match="the first argument of @option_group must be its name/title"
+    ):
+        @option_group(
+            option('--one'),
+            option('--two'),
+        )
+        def f(one, two):
+            pass
 
 
 @parametrize(
@@ -44,10 +61,13 @@ def test_option_group_constraints_are_checked(runner, get_example_command):
 
 
 def test_option_group_decorator_raises_if_group_is_passed_to_contained_option():
-    func = cloup.option_group(
-        'a group', cloup.option('--opt', group=cloup.OptionGroup('another group')))
     with pytest.raises(ValueError):
-        func(noop)
+        @option_group(
+            'a group',
+            cloup.option('--opt', group=OptionGroup('another group'))
+        )
+        def f(opt):
+            pass
 
 
 def test_option_group_decorator_raises_for_no_options():
@@ -55,18 +75,26 @@ def test_option_group_decorator_raises_for_no_options():
         cloup.option_group('grp')
 
 
-@pytest.mark.parametrize(['ctx_value', 'cmd_value', 'should_align'], [
-    pytest.param(True, None, True, id='ctx-yes'),
-    pytest.param(False, None, False, id='ctx-no'),
-    pytest.param(False, True, True, id='none'),
-    pytest.param(True, False, False, id='ctx-yes_cmd-no'),
-    pytest.param(False, True, True, id='ctx-no_cmd-yes'),
-])
-def test_align_option_groups_context_setting(runner, ctx_value, cmd_value, should_align):
-    @cloup.command(
-        context_settings=dict(align_option_groups=ctx_value),
+@pytest.mark.parametrize(
+    'cmd_value', [MISSING, None, True, False],
+    ids=lambda val: f'cmd_{val}'
+)
+@pytest.mark.parametrize(
+    'ctx_value', [MISSING, None, True, False],
+    ids=lambda val: f'ctx_{val}'
+)
+def test_align_option_groups_context_setting(runner, ctx_value, cmd_value):
+    should_align = pick_first_bool([cmd_value, ctx_value], default=True)
+    cxt_settings = pick_non_missing(dict(
+        align_option_groups=ctx_value,
+        terminal_width=80,
+    ))
+    cmd_kwargs = pick_non_missing(dict(
         align_option_groups=cmd_value,
-    )
+        context_settings=cxt_settings
+    ))
+
+    @cloup.command(**cmd_kwargs)
     @cloup.option_group('First group', option('--opt', help='first option'))
     @cloup.option_group('Second group', option('--much-longer-opt', help='second option'))
     @pass_context
@@ -152,3 +180,97 @@ def test_option_group_options_setter_set_the_hidden_attr_of_options():
     group.hidden = True
     group.options = opts
     assert all(opt.hidden for opt in opts)
+
+
+def test_option_group_with_constrained_subgroups(runner):
+    @cloup.command()
+    @option_group(
+        "Some options",
+        RequireAtLeast(1)(
+            option('-a', is_flag=True),
+            option('-b', is_flag=True)
+        ),
+        mutually_exclusive(
+            option('-c', is_flag=True),
+            option('-d', is_flag=True),
+        ),
+        option('-e', is_flag=True),
+    )
+    def cmd(a, b, c, d, e):
+        pass
+
+    cmd = cast(cloup.Command, cmd)
+    assert len(cmd.option_groups) == 1
+    assert len(cmd.option_groups[0]) == 5
+
+    assert runner.invoke(cmd, ['-abc']).exit_code == 0
+    assert 'Error: at least 1' in runner.invoke(cmd, ['-c']).output
+    assert 'mutually exclusive' in runner.invoke(cmd, ['-acd']).output
+
+    expected_help = dedent("""
+        Usage: cmd [OPTIONS]
+
+        Some options:
+          -a
+          -b
+          -c
+          -d
+          -e
+
+        Other options:
+          --help  Show this message and exit.
+    """).lstrip()
+    actual_help = runner.invoke(cmd, ['--help']).output
+    assert actual_help == expected_help
+
+
+def test_usage_of_constraints_as_decorators_inside_option_group(runner):
+    @cloup.command()
+    @cloup.option_group(
+        "Options",
+        mutually_exclusive(
+            cloup.option('-a', is_flag=True),
+            cloup.option('-b', is_flag=True),
+            cloup.option('-c', is_flag=True),
+        ),
+        cloup.option('-d', is_flag=True),
+    )
+    def cmd(a, b, c, d):
+        pass
+
+    expected_help = dedent("""
+        Usage: cmd [OPTIONS]
+
+        Options:
+          -a
+          -b
+          -c
+          -d
+
+        Other options:
+          --help  Show this message and exit.
+    """).lstrip()
+
+    res = runner.invoke(cmd, args=['--help'])
+    assert res.output == expected_help
+
+    assert runner.invoke(cmd, args=[]).exit_code == 0
+    assert runner.invoke(cmd, args='-d'.split()).exit_code == 0
+    assert runner.invoke(cmd, args='-ad'.split()).exit_code == 0
+
+    res = runner.invoke(cmd, args='-ab'.split())
+    assert res.exit_code == click.UsageError.exit_code
+    assert 'mutually exclusive' in res.output
+
+
+def test_option_groups_raises_if_input_decorator_add_an_argument():
+    with pytest.raises(
+        TypeError, match='only parameter of type `Option` can be added to option groups'
+    ):
+        option_group(
+            'Title',
+            mutually_exclusive(
+                cloup.argument('arg'),
+                cloup.option('--opt')
+            )
+        )(new_dummy_func())
