@@ -7,7 +7,9 @@ from click import pass_context
 
 import cloup
 from cloup import Section
-from tests.util import noop
+from cloup._util import pick_non_missing
+from cloup.typing import MISSING
+from tests.util import new_dummy_func, pick_first_bool
 
 
 @pytest.mark.parametrize(
@@ -25,7 +27,7 @@ def test_subcommand_sections_are_correctly_rendered_in_help(
 
 
 @pytest.mark.parametrize(
-    'subcommand_cls', [click.Command, cloup.Command,  click.Group, cloup.Group],
+    'subcommand_cls', [click.Command, cloup.Command, click.Group, cloup.Group],
     ids=['click_Command', 'cloup_Command', 'click_Group', 'cloup_Group'],
 )
 @pytest.mark.parametrize(
@@ -46,7 +48,7 @@ def test_Group_subcommand_decorator(subcommand_cls, assign_to_section):
         section=section_arg,
         cls=subcommand_cls,
         help='Help'
-    )(noop)
+    )(new_dummy_func())
     assert subcommand.__class__ is subcommand_cls
     assert subcommand.help == 'Help'
     assert grp.commands[subcommand_name] is subcommand
@@ -58,30 +60,38 @@ def test_Group_subcommand_decorator(subcommand_cls, assign_to_section):
         assert grp._default_section.commands[subcommand_name] is subcommand
 
 
-@pytest.mark.parametrize(['ctx_value', 'cmd_value', 'should_align'], [
-    pytest.param(True,  None,  True,  id='ctx-yes'),
-    pytest.param(False, None,  False, id='ctx-no'),
-    pytest.param(False, True,  True,  id='none'),
-    pytest.param(True,  False, False, id='ctx-yes_cmd-no'),
-    pytest.param(False, True,  True,  id='ctx-no_cmd-yes'),
-])
-def test_align_sections_context_setting(runner, ctx_value, cmd_value, should_align):
-    @cloup.group(
-        context_settings=dict(align_sections=ctx_value),
+@pytest.mark.parametrize(
+    'cmd_value', [MISSING, None, True, False],
+    ids=lambda val: f'cmd_{val}'
+)
+@pytest.mark.parametrize(
+    'ctx_value', [MISSING, None, True, False],
+    ids=lambda val: f'ctx_{val}'
+)
+def test_align_sections_context_setting(runner, ctx_value, cmd_value):
+    should_align = pick_first_bool([cmd_value, ctx_value], default=True)
+    cxt_settings = pick_non_missing(dict(
+        align_sections=ctx_value,
+        terminal_width=80,
+    ))
+    cmd_kwargs = pick_non_missing(dict(
         align_sections=cmd_value,
-    )
+        context_settings=cxt_settings
+    ))
+
+    @cloup.group(**cmd_kwargs)
     @pass_context
     def cmd(ctx, one, much_longer_opt):
         assert cmd.must_align_sections(ctx) == should_align
 
     cmd.section(
         "First section",
-        cloup.command('cmd', help='First command help')(noop),
+        cloup.command('cmd', help='First command help')(new_dummy_func()),
     )
 
     cmd.section(
         "Second section",
-        cloup.command('longer-cmd', help='Second command help')(noop),
+        cloup.command('longer-cmd', help='Second command help')(new_dummy_func()),
     )
 
     result = runner.invoke(cmd, args=('--help',))
@@ -104,3 +114,37 @@ def test_align_sections_context_setting(runner, ctx_value, cmd_value, should_ali
     expected = dedent(expected).strip()
     end = start + len(expected)
     assert result.output[start:end] == expected
+
+
+def test_override_format_subcommand_name(runner):
+    class MyGroup(cloup.Group):
+        def format_subcommand_name(self, name: str, cmd: click.Command) -> str:
+            return '*special*' if name == 'special' else name
+
+    main = MyGroup(name='main')
+    main.section(
+        'Commands',
+        cloup.Command(name='special', help='A special command.'),
+        cloup.Command(name='ordinary', help='An ordinary command.')
+    )
+
+    res = runner.invoke(main, ['--help'])
+    expected_help = dedent("""
+        Usage: main [OPTIONS] COMMAND [ARGS]...
+
+        Options:
+          --help  Show this message and exit.
+
+        Commands:
+          *special*  A special command.
+          ordinary   An ordinary command.
+    """)[1:]
+    assert res.output == expected_help
+
+
+def test_section_error_if_first_arg_is_not_a_string():
+    with pytest.raises(TypeError, match="the first argument must be a string"):
+        Section([cloup.Command('cmd')])
+    grp = cloup.Group()
+    with pytest.raises(TypeError, match="the first argument must be a string"):
+        grp.section([cloup.Command('cmd')])
